/***********************************************************************
 * cubixapi.CPP
 *  
 *    API for Cubix Direct X Demo
 *
 *
 * Toby Opferman Copyright (c) 2002
 *
 ***********************************************************************/
 

                                                                     
#include <windows.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <ddraw.h>


#include "cubixapi.h"


 /***********************************************************************
  * Internal Structures
  ***********************************************************************/
  
  typedef struct {

      double X;
      double Y;
      double Z;
      char  Colour;

  } POINT3D, *PPOINT3D;

  typedef struct {

      int X;
      int Y;
      char  Colour;

  } POINT2D, *PPOINT2D;

  typedef struct {

      int Index1;
      int Index2;
      int Index3;
      int Index4;

  } FACE, *PFACE;

  typedef struct {

      FACE Faces[6];
      
      POINT3D Points[8];
      POINT3D RotatedPoints[8];
      POINT3D WorldCoordinates;
      POINT3D Camera;
      POINT2D TwoDeePoints[8];

      double y_rotation;
      double x_rotation;
      double z_rotation;

      double y_radians;
      double x_radians;
      double z_radians;


  } CUBE3D, *PCUBE3D;

  typedef struct {

      HWND hWnd;
      LPDIRECTDRAW lpDirectDraw;
      LPDIRECTDRAWSURFACE lpDirectDrawSurface;
      LPDIRECTDRAWSURFACE lpDirectBackSurface;
      DDSURFACEDESC ddSurfaceDesc;
      CUBE3D Cube3D;

  } CUBIX, *PCUBIX;



 /***********************************************************************
  * Globals used by Drawing functions for screen calculation
  ***********************************************************************/
  unsigned int g_ScreenWidth;
  unsigned int g_ScreenPitch;
  unsigned int g_ScreenHeight;
  BOOL g_BackFaceRemovalOff = FALSE;

  #define ASPECT 1
  #define VIEW   256

 /***********************************************************************
  * Internal APIs
  ***********************************************************************/
  void CubixApi_DrawLine2d(UCHAR *pVideoBuffer, PPOINT2D pP1, PPOINT2D pP2);
  void CubixApi_DrawTriangle2d();
  void CubixApi_DrawFlatBottomTriangle2d();
  void CubixApi_DrawFlatTopTriangle2d();
  void CubixApi_AddPoints3d(PPOINT3D pP1, PPOINT3D pP2, PPOINT3D pAnswer);
  void CubixApi_SubPoints3d(PPOINT3D pP1, PPOINT3D pP2, PPOINT3D pAnswer);
  void CubixApi_ComputeCrossProduct3d(PPOINT3D pP1, PPOINT3D pP2, PPOINT3D pAnswer);
  void CubixApi_ComputeNormal3d(PPOINT3D pP1, PPOINT3D pP2, PPOINT3D pP3, PPOINT3D pNormal);
  inline void CubixApi_PlotPixel2d(UCHAR *pVideoBuffer, PPOINT2D pP1);
  inline void CubixApi_Project3dTo2d(PPOINT3D pP3D, PPOINT2D pP2D);
  void CubixApi_InitCube(PCUBE3D pCube3d);
  inline void CubixApi_RotatePointsX(PPOINT3D pPoint, PPOINT3D pRPoint, double radians);
  inline void CubixApi_RotatePointsY(PPOINT3D pPoint, PPOINT3D pRPoint, double radians);
  inline void CubixApi_RotatePointsZ(PPOINT3D pPoint, PPOINT3D pRPoint, double radians);
  void CubixApi_RotatePoints(PCUBE3D pCube3d);
  void CubixApi_DrawCube(UCHAR *pVideoBuffer, PCUBE3D pCube3d);
  void CubixApi_ProjectPoints(PCUBE3D pCube3d);
  void CubixApi_Debug(char *pszFormatString, ...);
  double CubixApi_ComputeDotProduct3d(PPOINT3D pP1, PPOINT3D pP2);
  void CubixApi_NormalizeVector(PPOINT3D pP1, PPOINT3D pNormal);
  



 /***********************************************************************
  * CubixApi_Init
  *  
  *    Initialize Cubix Graphics
  *
  * Parameters
  *     Window  Handle
  * 
  * Return Value
  *     Handle To Cubix
  *
  ***********************************************************************/
HCUBIX WINAPI CubixApi_Init(HWND hWnd)
{
    PCUBIX pCubix = NULL;

    if(pCubix = (PCUBIX)LocalAlloc(LMEM_ZEROINIT, sizeof(CUBIX)))
    {
       pCubix->hWnd = hWnd;

       if(SUCCEEDED(DirectDrawCreate(NULL, &pCubix->lpDirectDraw, NULL)))
       {
           pCubix->lpDirectDraw->Initialize(NULL);
           
           pCubix->lpDirectDraw->SetCooperativeLevel(hWnd, DDSCL_FULLSCREEN |
                                                           DDSCL_ALLOWMODEX |
                                                           DDSCL_EXCLUSIVE  |
                                                           DDSCL_ALLOWREBOOT);
           pCubix->lpDirectDraw->SetDisplayMode(320, 200,  8);

           g_ScreenWidth  = 320;
           g_ScreenHeight = 200;

           pCubix->ddSurfaceDesc.dwSize            = sizeof(pCubix->ddSurfaceDesc);
           pCubix->ddSurfaceDesc.dwFlags           = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
           pCubix->ddSurfaceDesc.ddsCaps.dwCaps    = DDSCAPS_PRIMARYSURFACE | DDSCAPS_COMPLEX | DDSCAPS_FLIP;
           pCubix->ddSurfaceDesc.dwBackBufferCount = 1;

           
           CubixApi_InitCube(&pCubix->Cube3D);

           if(FAILED(pCubix->lpDirectDraw->CreateSurface(&pCubix->ddSurfaceDesc, &pCubix->lpDirectDrawSurface, NULL)))
           {
               CubixApi_UnInit((HCUBIX)pCubix);
               pCubix = NULL;
           }
           else
           {
               pCubix->ddSurfaceDesc.ddsCaps.dwCaps    = DDSCAPS_BACKBUFFER;
               
               if(FAILED(pCubix->lpDirectDrawSurface->GetAttachedSurface(&pCubix->ddSurfaceDesc.ddsCaps, &pCubix->lpDirectBackSurface)))
               {
                   CubixApi_UnInit((HCUBIX)pCubix);
                   pCubix = NULL;
               }

           }

       }
       else
       {
           CubixApi_UnInit((HCUBIX)pCubix);
           pCubix = NULL;
       }
    }

    return (HCUBIX)pCubix;
}


 /***********************************************************************
  * CubixApi_UnInit
  *  
  *    UnInitialize Cubix Graphics
  *
  * Parameters
  *     Handle to Cubix
  * 
  * Return Value
  *     Nothing
  *
  ***********************************************************************/
void WINAPI CubixApi_UnInit(HCUBIX hCubix)
{
    PCUBIX pCubix = (PCUBIX)hCubix;

    if(pCubix)
    {

        if(pCubix->lpDirectDrawSurface)
        {
            pCubix->lpDirectDrawSurface->Release();
            pCubix->lpDirectDrawSurface = NULL;
        }

        if(pCubix->lpDirectDraw)
        {
            pCubix->lpDirectDraw->Release();
            pCubix->lpDirectDraw = NULL;
        }


        LocalFree(pCubix);

        pCubix = NULL;
    }
}


 /***********************************************************************
  * CubixApi_PerformDemo
  *  
  *    Display The Demo
  *
  * Parameters
  *     Handle to Cubix
  * 
  * Return Value
  *     Nothing
  *
  ***********************************************************************/
void WINAPI CubixApi_PerformDemo(HCUBIX hCubix)
{
   PCUBIX pCubix = (PCUBIX)hCubix;
   UCHAR  *pVideoBuffer;

   pCubix->ddSurfaceDesc.dwSize = sizeof(pCubix->ddSurfaceDesc);

   if(SUCCEEDED(pCubix->lpDirectBackSurface->Lock(NULL, &pCubix->ddSurfaceDesc, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT, NULL)))
   {
       
        g_ScreenPitch = pCubix->ddSurfaceDesc.lPitch;
        pVideoBuffer = (UCHAR *)pCubix->ddSurfaceDesc.lpSurface;
        
        for(unsigned int y = 0; y < g_ScreenHeight; y++)
          memset(&pVideoBuffer[y*g_ScreenPitch], 0, g_ScreenWidth);

        
        if(GetAsyncKeyState((int)'X') & 1)
        {
            if(pCubix->Cube3D.x_radians)
                pCubix->Cube3D.x_radians = 0;
            else
                pCubix->Cube3D.x_radians = (double)3.14*(1.0/180.0);
        }

        if(GetAsyncKeyState((int)'Y') & 1)
        {
            if(pCubix->Cube3D.y_radians)
                pCubix->Cube3D.y_radians = 0;
            else
                pCubix->Cube3D.y_radians = (double)3.14*(1.0/180.0);
        }

        if(GetAsyncKeyState((int)'Z') & 1)
        {
            if(pCubix->Cube3D.z_radians)
                pCubix->Cube3D.z_radians = 0;
            else
                pCubix->Cube3D.z_radians = (double)3.14*(1.0/180.0);
        }

        if(GetAsyncKeyState(VK_UP))
        {
            pCubix->Cube3D.WorldCoordinates.Y+=5;
        }

        if(GetAsyncKeyState(VK_DOWN))
        {
           pCubix->Cube3D.WorldCoordinates.Y-=5;
        }

        if(GetAsyncKeyState(VK_RIGHT))
        {
            pCubix->Cube3D.WorldCoordinates.X+=5;
        }

        if(GetAsyncKeyState(VK_LEFT))
        {
            pCubix->Cube3D.WorldCoordinates.X-=5;
        }

        if(GetAsyncKeyState(VK_SUBTRACT))
        {
            pCubix->Cube3D.WorldCoordinates.Z +=5;
        }

        if(GetAsyncKeyState(VK_ADD))
        {
            pCubix->Cube3D.WorldCoordinates.Z -=5;
        }

        if(GetAsyncKeyState(VK_SPACE))
        {
            g_BackFaceRemovalOff = !g_BackFaceRemovalOff;
        }


        CubixApi_DrawCube(pVideoBuffer, &pCubix->Cube3D);
        
        
        pCubix->lpDirectBackSurface->Unlock(NULL);

        while(FAILED(pCubix->lpDirectDrawSurface->Flip(NULL, DDFLIP_WAIT))) ;
   }

}



 /***********************************************************************
  * CubixApi_DrawCube
  *  
  *    Computes The Normal of the 3D plane
  *
  * Parameters
  *     Video buffer, Cube
  * 
  * Return Value
  *     Nothing
  *
  ***********************************************************************/
 void CubixApi_DrawCube(UCHAR *pVideoBuffer, PCUBE3D pCube3d)
 {
     int x;
     double DotProduct, Angle = (90.0/180.0)*3.14;

     CubixApi_RotatePoints(pCube3d);
     CubixApi_ProjectPoints(pCube3d);
     
     for(x = 0; x < 6; x++)
     {   
         POINT3D Point3d, p1, p2, view;
         
        // CubixApi_ComputeNormal3d(&pCube3d->RotatedPoints[pCube3d->Faces[x].Index1], &pCube3d->RotatedPoints[pCube3d->Faces[x].Index2], &pCube3d->RotatedPoints[pCube3d->Faces[x].Index3], &Point3d);

        // view.X = -pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].X;
        // view.Y = -pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].Y;
        /// view.Z = -pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].Z;

        // DotProduct = CubixApi_ComputeDotProduct3d(&view, &Point3d);

        // CubixApi_Debug("DrawCube: Face %i : Normal = (%1.2f, %1.2f, %1.2f) DotProduct = %1.2f < %1.2f angle\n", x, Point3d.X, Point3d.Y, Point3d.Z, DotProduct, Angle);

        // p1.X = pCube3d->RotatedPoints[pCube3d->Faces[x].Index1].X - pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].X;
        // p1.Y = pCube3d->RotatedPoints[pCube3d->Faces[x].Index1].Y - pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].Y;
        // p2.X = pCube3d->RotatedPoints[pCube3d->Faces[x].Index3].X - pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].X;
        // p2.Y = pCube3d->RotatedPoints[pCube3d->Faces[x].Index3].Y - pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].Y;
         
       //  if((DotProduct > Angle) || g_BackFaceRemovalOff)
         //if((DotProduct < ((270/180)*3.14) && DotProduct > ((90/180)*3.14)) || g_BackFaceRemovalOff)
         //if((p1.X*p2.Y - p1.Y*p2.X) > 0)

         DotProduct = (pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].X -
                       pCube3d->RotatedPoints[pCube3d->Faces[x].Index1].X) *

                      (pCube3d->RotatedPoints[pCube3d->Faces[x].Index3].Y -
                       pCube3d->RotatedPoints[pCube3d->Faces[x].Index1].Y) -

                      (pCube3d->RotatedPoints[pCube3d->Faces[x].Index2].Y -
                       pCube3d->RotatedPoints[pCube3d->Faces[x].Index1].Y) *

                      (pCube3d->RotatedPoints[pCube3d->Faces[x].Index3].X -
                       pCube3d->RotatedPoints[pCube3d->Faces[x].Index1].X);

         if(DotProduct < 0)
         {
            CubixApi_DrawLine2d(pVideoBuffer, &pCube3d->TwoDeePoints[pCube3d->Faces[x].Index1], &pCube3d->TwoDeePoints[pCube3d->Faces[x].Index2]);
            CubixApi_DrawLine2d(pVideoBuffer, &pCube3d->TwoDeePoints[pCube3d->Faces[x].Index2], &pCube3d->TwoDeePoints[pCube3d->Faces[x].Index3]);
            CubixApi_DrawLine2d(pVideoBuffer, &pCube3d->TwoDeePoints[pCube3d->Faces[x].Index3], &pCube3d->TwoDeePoints[pCube3d->Faces[x].Index4]);
            CubixApi_DrawLine2d(pVideoBuffer, &pCube3d->TwoDeePoints[pCube3d->Faces[x].Index4], &pCube3d->TwoDeePoints[pCube3d->Faces[x].Index1]);
         }
     }

 }

 /***********************************************************************
  * CubixApi_ComputeNormal3d
  *  
  *    Computes The Normal of the 3D plane
  *
  * Parameters
  *     Points In the plane, Point to return Normal in
  * 
  * Return Value
  *     Nothing
  *
  ***********************************************************************/
 void CubixApi_ComputeNormal3d(PPOINT3D pP1, PPOINT3D pP2, PPOINT3D pP3, PPOINT3D pNormal)
 {
     POINT3D Point1, Point2;

    CubixApi_SubPoints3d(pP1, pP2, &Point1);
    CubixApi_SubPoints3d(pP3, pP2, &Point2);

    CubixApi_ComputeCrossProduct3d(&Point1, &Point2, pNormal);
 }

 
 /***********************************************************************
 * CubixApi_DrawLine2d
 *  
 *    Draws a line in 2d, 
 *
 * Parameters
 *     Video Buffer, Points to draw line.  
 * 
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_DrawLine2d(UCHAR *pVideoBuffer, PPOINT2D pP1, PPOINT2D pP2)
 {
    POINT2D Point;
    int DeltaX, DeltaY, IncX = 1, IncY = 1, Error = 0, Index;

    CubixApi_Debug("Drawing Line (%i, %i) to (%i, %i)\n", pP1->X, pP1->Y, pP2->X, pP2->Y);

    Point = *pP1;
    
    DeltaX = pP2->X - pP1->X;
    DeltaY = pP2->Y - pP1->Y;

    if(DeltaX < 0)
    {
        DeltaX *= -1;
        IncX  = -1;
    }

    if(DeltaY < 0)
    {
        DeltaY *= -1;
        IncY = -1;
    }


    if(DeltaX > DeltaY)
    {

        for(Index = 0; Index <= DeltaX; Index++)
        {
           
           CubixApi_PlotPixel2d(pVideoBuffer, &Point);

           Error += DeltaY;

           if(DeltaX < Error)
           {
               Error -= DeltaX;
               Point.Y += IncY;
           }

           Point.X += IncX;

        }

    }
    else
    {
        for(Index = 0; Index <= DeltaY; Index++)
        {
           
           CubixApi_PlotPixel2d(pVideoBuffer, &Point);

           Error += DeltaX;

           if(Error > 0)
           {
               Error -= DeltaY;
               Point.X += IncX;
           }

           Point.Y += IncY;
        }

    }
 }

 
 /***********************************************************************
 * CubixApi_DrawTriangle2d
 *  
 *    Computes The Normal of the 3D plane
 *
 * Parameters
 *     Points In the plane, Point to return Normal in
 * 
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_DrawTriangle2d()
 {
 }

 /***********************************************************************
 * CubixApi_DrawFlatBottomTriangle2d
 *  
 *    Computes The Normal of the 3D plane
 *
 * Parameters
 *     Points In the plane, Point to return Normal in
 * 
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_DrawFlatBottomTriangle2d()
 {
 }

 /***********************************************************************
 * CubixApi_DrawFlatTopTriangle2d
 *  
 *    Computes The Normal of the 3D plane
 *
 * Parameters
 *     Points In the plane, Point to return Normal in
 * 
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_DrawFlatTopTriangle2d()
 {
 }

 /***********************************************************************
 * CubixApi_AddPoints3d
 *  
 *    Adds 2 3D Points
 *
 * Parameters
 *     Points to add, Point to return Answer in in
 * 
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_AddPoints3d(PPOINT3D pP1, PPOINT3D pP2, PPOINT3D pAnswer)
 {
     pAnswer->X = pP1->X + pP2->X;
     pAnswer->Y = pP1->Y + pP2->Y;
     pAnswer->Z = pP1->Z + pP2->Z;
 }

 /***********************************************************************
 * CubixApi_SubPoints3d
 *  
 *    Subtracts 2 3D Points
 *
 * Parameters
 *     Points to Subtrtact, Point to return Answer in in
 * 
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_SubPoints3d(PPOINT3D pP1, PPOINT3D pP2, PPOINT3D pAnswer)
 {
     pAnswer->X = pP1->X - pP2->X;
     pAnswer->Y = pP1->Y - pP2->Y;
     pAnswer->Z = pP1->Z - pP2->Z;
 }

 /***********************************************************************
 * CubixApi_ComputeCrossProduct3d
 *  
 *    Computes cross product
 *
 * Parameters
 *     Points to compute cross product, Point to return Answer in in
 * 
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_ComputeCrossProduct3d(PPOINT3D pP1, PPOINT3D pP2, PPOINT3D pAnswer)
 {
      pAnswer->X = pP1->Y * pP2->Z - pP1->Z * pP1->Y;
      pAnswer->Y = pP1->Z * pP2->X - pP1->X * pP2->Z;
      pAnswer->Z = pP1->X * pP2->Y - pP1->Y * pP2->X;
 }


 /***********************************************************************
 * CubixApi_ComputeDotProduct3d
 *  
 *    Computes cross product
 *
 * Parameters
 *     Points to compute dot product
 * 
 * Return Value
 *     Dot Product
 *
 ***********************************************************************/
 double CubixApi_ComputeDotProduct3d(PPOINT3D pP1, PPOINT3D pP2)
 {
      POINT3D One, Two;
      double angle;
      BOOL bDone = FALSE;

      CubixApi_NormalizeVector(pP1, &One);
      CubixApi_NormalizeVector(pP2, &Two);

      angle = One.X * Two.X + One.Y * Two.Y + One.Z * Two.Z;

      while(!bDone)
      {
         bDone = TRUE;

         if(angle > 2*3.14)
         {
            bDone = FALSE;
            angle -= (2*3.14);
         }

         if(angle < 0)
         {
            bDone = FALSE;
            angle += (2*3.14);
         }
      }

      return angle;
 }




 /***********************************************************************
 * CubixApi_PlotPixel2d
 *  
 *    Plots a pixel in 2d
 *
 *    This function performs clipping to the screen.
 *
 * Parameters
 *     Video buffer, Point to plot
 * 
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_PlotPixel2d(UCHAR *pVideoBuffer, PPOINT2D pP1)
 {
     if(((unsigned int)pP1->Y) < g_ScreenHeight && ((unsigned int)pP1->X) < g_ScreenWidth)
        pVideoBuffer[pP1->X + pP1->Y*g_ScreenPitch] = pP1->Colour;
 }

 
 
  /***********************************************************************
 * CubixApi_Project3dTo2d
 *  
 *    Projects a 3D point to 2D.
 *
 *    
 *
 * Parameters
 *     3D Point, 2D Point
 *
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_Project3dTo2d(PPOINT3D pP3D, PPOINT2D pP2D)
 {
   static int HalfScreenHeight = g_ScreenHeight/2;
   static int HalfScreenWidth  = g_ScreenWidth/2;
   
   if(pP3D->Z)
   {
      pP2D->X = (int)(pP3D->X*VIEW/pP3D->Z + HalfScreenWidth);
      pP2D->Y = (int)(HalfScreenHeight - pP3D->Y*ASPECT*VIEW/pP3D->Z);
   }
   else
   {
       pP2D->X = (int)pP3D->X + HalfScreenWidth;
       pP2D->Y = (int)pP3D->Y + HalfScreenHeight;
   }

   CubixApi_Debug("Projecting 3D %1.2f, %1.2f, %1.2f to 2D %i, %i\n", pP3D->X, pP3D->Y, pP3D->Z, pP2D->X, pP2D->Y);

   pP2D->Colour = pP3D->Colour;
 }


 /***********************************************************************
* CubixApi_InitCube
*  
*    Initializes The 3D Cube
*
*    
*
* Parameters
*     3D Cube
*
* Return Value
*     Nothing
*
***********************************************************************/
 void CubixApi_InitCube(PCUBE3D pCube3d)
 {

     pCube3d->Points[0].X = -10;
     pCube3d->Points[0].Y = 10;
     pCube3d->Points[0].Z = 10;
     pCube3d->Points[0].Colour = 4;

     pCube3d->Points[1].X = 10;
     pCube3d->Points[1].Y = 10;
     pCube3d->Points[1].Z = 10;
     pCube3d->Points[1].Colour = 4;

     pCube3d->Points[2].X = -10;
     pCube3d->Points[2].Y = 10;
     pCube3d->Points[2].Z = -10;
     pCube3d->Points[2].Colour = 4;

     pCube3d->Points[3].X = 10;
     pCube3d->Points[3].Y = 10;
     pCube3d->Points[3].Z = -10;
     pCube3d->Points[3].Colour = 4;

     pCube3d->Points[4].X = -10;
     pCube3d->Points[4].Y = -10;
     pCube3d->Points[4].Z = 10;
     pCube3d->Points[4].Colour = 4;

     pCube3d->Points[5].X = 10;
     pCube3d->Points[5].Y = -10;
     pCube3d->Points[5].Z = 10;
     pCube3d->Points[5].Colour = 4;

     pCube3d->Points[6].X = -10;
     pCube3d->Points[6].Y = -10;
     pCube3d->Points[6].Z = -10;
     pCube3d->Points[6].Colour = 4;

     pCube3d->Points[7].X = 10;
     pCube3d->Points[7].Y = -10;
     pCube3d->Points[7].Z = -10;
     pCube3d->Points[7].Colour = 4;

     pCube3d->Faces[0].Index1 = 0;
     pCube3d->Faces[0].Index2 = 2;
     pCube3d->Faces[0].Index3 = 3;
     pCube3d->Faces[0].Index4 = 1;

     pCube3d->Faces[1].Index1 = 0;
     pCube3d->Faces[1].Index2 = 1;
     pCube3d->Faces[1].Index3 = 5;
     pCube3d->Faces[1].Index4 = 4;

     pCube3d->Faces[2].Index1 = 4;
     pCube3d->Faces[2].Index2 = 5;
     pCube3d->Faces[2].Index3 = 7;
     pCube3d->Faces[2].Index4 = 6;

     pCube3d->Faces[3].Index1 = 1;
     pCube3d->Faces[3].Index2 = 3;
     pCube3d->Faces[3].Index3 = 7;
     pCube3d->Faces[3].Index4 = 5;

     pCube3d->Faces[4].Index1 = 2;
     pCube3d->Faces[4].Index2 = 0;
     pCube3d->Faces[4].Index3 = 4;
     pCube3d->Faces[4].Index4 = 6;

     pCube3d->Faces[5].Index1 = 3;
     pCube3d->Faces[5].Index2 = 2;
     pCube3d->Faces[5].Index3 = 6;
     pCube3d->Faces[5].Index4 = 7
         ;
     pCube3d->WorldCoordinates.X = 0;
     pCube3d->WorldCoordinates.Y = 0;
     pCube3d->WorldCoordinates.Z = 300;

     pCube3d->x_rotation = (double)3.14*(1.0/180.0);
     pCube3d->y_rotation = (double)3.14*(1.0/180.0);
     pCube3d->z_rotation = (double)3.14*(1.0/180.0);

     pCube3d->x_radians = (double)3.14*(1.0/180.0);
     pCube3d->y_radians = (double)3.14*(1.0/180.0);
     pCube3d->z_radians = (double)3.14*(1.0/180.0);


 }



 


 /***********************************************************************
* CubixApi_RotatePoints
*  
*    Rotates The Points
*
*    
*
* Parameters
*     3D Cube
*
* Return Value
*     Nothing
*
***********************************************************************/
 void CubixApi_RotatePoints(PCUBE3D pCube3d)
 {
     int Index;
     POINT3D Points;

     for(Index = 0; Index < 8; Index++)
     {
         pCube3d->Points[Index].X;
         pCube3d->Points[Index].Y;
         pCube3d->Points[Index].Z;

         Points = pCube3d->Points[Index];

         CubixApi_RotatePointsX(&Points, &pCube3d->RotatedPoints[Index], pCube3d->x_rotation);

         Points = pCube3d->RotatedPoints[Index];

         CubixApi_RotatePointsY(&Points, &pCube3d->RotatedPoints[Index], pCube3d->y_rotation);

         Points = pCube3d->RotatedPoints[Index];

         CubixApi_RotatePointsZ(&Points, &pCube3d->RotatedPoints[Index], pCube3d->z_rotation);

     }

     pCube3d->x_rotation += pCube3d->x_radians;
     pCube3d->y_rotation += pCube3d->y_radians;
     pCube3d->z_rotation += pCube3d->z_radians;

 }





 /***********************************************************************
* CubixApi_RotatePointsX
*  
*    Rotates The Points
*
*    
*
* Parameters
*     3D Cube
*
* Return Value
*     Nothing
*
***********************************************************************/
void CubixApi_RotatePointsX(PPOINT3D pPoint, PPOINT3D pRPoint, double radians)
{
    pRPoint->X = pPoint->X;
    pRPoint->Y = pPoint->Y*cos(radians) + pPoint->Z*sin(radians);
    pRPoint->Z = pPoint->Y*sin(radians) - pPoint->Z*cos(radians);
    pRPoint->Colour = pPoint->Colour;
}



 /***********************************************************************
* CubixApi_RotatePointsY
*  
*    Rotates The Points
*
*    
*
* Parameters
*     3D Cube
*
* Return Value
*     Nothing
*
***********************************************************************/
void CubixApi_RotatePointsY(PPOINT3D pPoint, PPOINT3D pRPoint, double radians)
{
    pRPoint->Y = pPoint->Y;
    pRPoint->X = pPoint->X*cos(radians) + pPoint->Z*sin(radians);
    pRPoint->Z = pPoint->X*sin(radians) - pPoint->Z*cos(radians);
    pRPoint->Colour = pPoint->Colour;
}



 /***********************************************************************
* CubixApi_RotatePointsZ
*  
*    Rotates The Points
*
*    
*
* Parameters
*     3D Cube
*
* Return Value
*     Nothing
*
***********************************************************************/
void CubixApi_RotatePointsZ(PPOINT3D pPoint, PPOINT3D pRPoint, double radians)
{
    pRPoint->Z = pPoint->Z;
    pRPoint->X = pPoint->X*cos(radians) + pPoint->Y*sin(radians);
    pRPoint->Y = pPoint->X*sin(radians) - pPoint->Y*cos(radians);
    pRPoint->Colour = pPoint->Colour;

}


 /***********************************************************************
* CubixApi_ProjectPoints
*  
*    Project 3D to 2D
*
*    
*
* Parameters
*     3D Cube
*
* Return Value
*     Nothing
*
***********************************************************************/
 void CubixApi_ProjectPoints(PCUBE3D pCube3d)
 {
     int Index;
     double x, y, z;

     x =  pCube3d->WorldCoordinates.X - pCube3d->Camera.X;
     y =  pCube3d->WorldCoordinates.Y - pCube3d->Camera.Y;
     z =  pCube3d->WorldCoordinates.Z - pCube3d->Camera.Z;

     for(Index = 0; Index < 8; Index++)
     {
        pCube3d->RotatedPoints[Index].X += x;
        pCube3d->RotatedPoints[Index].Y += y;
        pCube3d->RotatedPoints[Index].Z += z;

        CubixApi_Project3dTo2d(&pCube3d->RotatedPoints[Index], &pCube3d->TwoDeePoints[Index]);
     }

 }


 /***********************************************************************
 * CubixApi_Debug
 *  
 *    Debug Shit
 *
 *    
 *
 * Parameters
 *     Debug
 *
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_Debug(char *pszFormatString, ...)
 {
     char DebugString[256];
     va_list vl;

     va_start(vl, pszFormatString);
     vsprintf(DebugString, pszFormatString, vl);
     va_end(vl);

     OutputDebugString(DebugString);
 }


 /***********************************************************************
 * CubixApi_NormalizeVector
 *  
 *    Normalize Vector
 *
 *    
 *
 * Parameters
 *     Vector, Normal
 *
 * Return Value
 *     Nothing
 *
 ***********************************************************************/
 void CubixApi_NormalizeVector(PPOINT3D pP1, PPOINT3D pNormal)
 {
     double Length = sqrt(pP1->X*pP1->X + pP1->Y*pP1->Y + pP1->Z*pP1->Z);

     pNormal->X = pP1->X / Length;
     pNormal->Y = pP1->Y / Length;
     pNormal->Z = pP1->Z / Length;
 }

_int64 one, two, three, four;

void CubixApi_Add(_int64 x, _int64 y)
{
   one = x + y;
   two = x - y;
   three = x * y;
   four = x / y;
}
  
