/***********************************************************************
 * cubixwinproc.CPP
 *  
 *    Window for Cubix Direct X Demo
 *
 *
 * Toby Opferman Copyright (c) 2002
 *
 ***********************************************************************/
 

                                                                     
#include <windows.h>
#include "cubixwinproc.h"
#include "cubixapi.h"


 /***********************************************************************
  * Internal APIs
  ***********************************************************************/
 LRESULT CALLBACK CubixWinProc_WindowProc(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam);


 /***********************************************************************
  * CubixWinProc_RegisterClass
  *  
  *    Registers the Cubix Window Proc Class
  *
  * Parameters
  *     instance Handle
  * 
  * Return Value
  *     TRUE on Success, FALSE on Failure
  *
  ***********************************************************************/
 BOOL WINAPI CubixWinProc_RegisterClass(HINSTANCE hInstance)
 {
     char *p = "hello";
     char x[] = "hello";
     WNDCLASSEX WndClassEx = {0};

     *p = 'd';
     *x = 'l';

     WndClassEx.cbSize        = sizeof(WndClassEx);
     WndClassEx.style         = 0;
     WndClassEx.lpfnWndProc   = CubixWinProc_WindowProc;
     WndClassEx.cbWndExtra    = 0;
     WndClassEx.hInstance     = hInstance;
     WndClassEx.hIcon         = NULL;
     WndClassEx.hCursor       = NULL;
     WndClassEx.hbrBackground = NULL;
     WndClassEx.lpszMenuName  = NULL;
     WndClassEx.lpszClassName = CUBIX_CLASS_NAME;
     WndClassEx.hIconSm       = NULL;
     
     return RegisterClassEx(&WndClassEx);
 }



 /***********************************************************************
  * CubixWinProc_CreateWindow
  *  
  *    Creates The Cubix Window
  *
  * Parameters
  *     instance Handle
  * 
  * Return Value
  *     Window Handle on Success, NULL on Failure
  *
  ***********************************************************************/
 HWND WINAPI CubixWinProc_CreateWindow(HINSTANCE hInstance)
 {
     HWND hWnd = NULL;

     if(!FindWindow(CUBIX_CLASS_NAME, NULL))
     {
        if(hWnd = CreateWindowEx(0, CUBIX_CLASS_NAME, "Cubix Demo", WS_POPUP | WS_VISIBLE, 0,  0, 0, 0, HWND_DESKTOP, NULL,  hInstance, NULL))
        {
            ShowWindow(hWnd, SW_HIDE);
            UpdateWindow(hWnd);
        }
     }

     return hWnd;
 }


 /***********************************************************************
  * CubixWinProc_MessageLoop
  *  
  *    Handles Window Messages to be dispatched to the Cubix Window
  *
  * Parameters
  *     instance Handle, Window Handle
  * 
  * Return Value
  *     Exit Value
  *
  ***********************************************************************/
 int WINAPI CubixWinProc_MessageLoop(HINSTANCE hInstance, HWND hWnd)
 {
     MSG Msg = {0};
     HCUBIX hCubix;
     BOOL bMessageLoop = TRUE;

     if(hCubix = CubixApi_Init(hWnd))
     {
        while(bMessageLoop)
        {
         if(PeekMessage(&Msg, NULL, 0, 0, PM_NOREMOVE))
         {
           if(Msg.message == WM_QUIT)
           {
              bMessageLoop = FALSE;
           }
           else
           {
              PeekMessage(&Msg,NULL, 0, 0, PM_REMOVE);
              TranslateMessage(&Msg);
              DispatchMessage(&Msg);
           }
         }
         else
         {
             CubixApi_PerformDemo(hCubix);
         }
        }

        CubixApi_UnInit(hCubix);
     }
     else
     {
         MessageBox(HWND_DESKTOP, "Failed To Initialize Direct X", "FAILURE", MB_OK | MB_ICONEXCLAMATION);
     }

     return Msg.wParam;
 }



 /***********************************************************************
  * CubixWinProc_WindowProc
  *  
  *    Handles Window Messages Send to the window
  *
  * Parameters
  *     Standard Windows Proc Handler
  * 
  * Return Value
  *     Exit Value
  *
  ***********************************************************************/
 LRESULT CALLBACK CubixWinProc_WindowProc(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
 {
     LRESULT dwRetValue = 0;

     switch(wMsg)
     {
         case WM_CREATE:
              break;

		 case WM_KEYDOWN:
         case WM_CLOSE:
              DestroyWindow(hWnd);
			  break;

		 case WM_DESTROY:
  		      PostQuitMessage(0);

         default:
              dwRetValue = DefWindowProc(hWnd, wMsg, wParam, lParam);
     }

     return dwRetValue;
 }


